@@ -36,6 +36,7 @@
 
 
 from .processor import ContentProcessor
+from .forward import ForwardContentProcessor
 from .command import CommandProcessor
 from .history import HistoryCommandProcessor, GroupCommandProcessor
 from .invite import InviteCommandProcessor
@@ -49,9 +50,13 @@ from .profile import ProfileCommandProcessor
 
 
 __all__ = [
-    'ContentProcessor', 'CommandProcessor',
+    'ContentProcessor',
+    'ForwardContentProcessor',
 
-    'HistoryCommandProcessor', 'GroupCommandProcessor',
+    'CommandProcessor',
+
+    'HistoryCommandProcessor',
+    'GroupCommandProcessor',
     'InviteCommandProcessor', 'ExpelCommandProcessor', 'QuitCommandProcessor',
     'ResetCommandProcessor', 'QueryCommandProcessor',
 
