@@ -37,28 +37,27 @@
 
 import weakref
 from abc import abstractmethod
-from typing import Optional
+from typing import Optional, Union
 
-from dimp import SymmetricKey, ID, Meta, LocalUser
+from dimp import SymmetricKey, ID, Meta, User
 from dimp import InstantMessage, SecureMessage, ReliableMessage
-from dimp import Content, ForwardContent, FileContent
-from dimp import GroupCommand, InviteCommand
+from dimp import Content, FileContent
 from dimp import Transceiver
 
-from .cpu import ContentProcessor
-
 from .delegate import Callback, CompletionHandler
 from .delegate import MessengerDelegate, ConnectionDelegate
 from .facebook import Facebook
 
+from .cpu import ContentProcessor
+
 
 class Messenger(Transceiver, ConnectionDelegate):
 
     def __init__(self):
         super().__init__()
-        self.__delegate: weakref.ReferenceType = None
         self.__context: dict = {}
-        self.__cpu: ContentProcessor = None
+        self.__delegate: weakref.ReferenceType = None
+        self.__cpu = ContentProcessor(messenger=self)
 
     #
     #   Delegate for sending data
@@ -70,10 +69,8 @@ class Messenger(Transceiver, ConnectionDelegate):
 
     @delegate.setter
     def delegate(self, value: Optional[MessengerDelegate]):
-        if value is None:
-            self.__delegate = None
-        else:
-            self.__delegate = weakref.ref(value)
+        assert value is not None, 'message delegate should not be empty'
+        self.__delegate = weakref.ref(value)
 
     #
     #   Environment variables as context
@@ -102,74 +99,66 @@ class Messenger(Transceiver, ConnectionDelegate):
             assert isinstance(barrack, Facebook), 'messenger delegate error: %s' % barrack
         return barrack
 
-    #
-    #   All local users (for decrypting received message)
-    #
-    @property
-    def local_users(self) -> Optional[list]:
-        return self.get_context('local_users')
-
-    @local_users.setter
-    def local_users(self, value: Optional[list]):
-        self.set_context('local_users', value)
-
-    #
-    #   Current user (for signing and sending message)
-    #
-    @property
-    def current_user(self) -> Optional[LocalUser]:
-        users = self.local_users
-        if users is not None and len(users) > 0:
+    def __select(self, receiver: ID) -> Optional[User]:
+        facebook = self.facebook
+        users = facebook.local_users
+        if users is None or len(users) == 0:
+            raise LookupError('local users should not be empty')
+        elif receiver.is_broadcast:
+            # broadcast message can decrypt by anyone, so just return current user
             return users[0]
+        if receiver.type.is_group():
+            # group message (recipient not designated)
+            members = facebook.members(identifier=receiver)
+            if members is None or len(members) == 0:
+                # TODO: query group members
+                #       (do it by application)
+                return None
+            for item in users:
+                if item.identifier in members:
+                    # set this item to be current user?
+                    return item
+        else:
+            # 1. personal message
+            # 2. split group message
+            assert receiver.type.is_user(), 'receiver ID error: %s' % receiver
+            for item in users:
+                if item.identifier == receiver:
+                    # set this item to be current user?
+                    return item
 
-    @current_user.setter
-    def current_user(self, value: LocalUser):
-        users = self.local_users
-        if users is None:
-            # local_users not set
-            self.set_context('local_users', [value])
-            return
-        elif len(users) == 0:
-            # local_users empty
-            users.append(value)
-            return
-        # search all users
-        for index, item in enumerate(users):
-            if item == value:
-                # got it
-                if index > 0:
-                    # move this user to the front
-                    users.pop(index)
-                    users.insert(0, value)
-                return
-        # user not exists, insert into the front
-        users.insert(0, value)
-
-    #
-    #   Content Processing Units
-    #
-    def cpu(self) -> ContentProcessor:
-        if self.__cpu is None:
-            self.__cpu = ContentProcessor(messenger=self)
-        return self.__cpu
+    def __trim(self, msg: SecureMessage) -> Optional[SecureMessage]:
+        receiver = self.facebook.identifier(msg.envelope.receiver)
+        user = self.__select(receiver=receiver)
+        if user is None:
+            # current users not match
+            msg = None
+        elif receiver.type.is_group():
+            # trim group message
+            msg = msg.trim(member=user.identifier)
+        return msg
 
     #
     #  Transform
     #
     def verify_message(self, msg: ReliableMessage) -> Optional[SecureMessage]:
+        facebook = self.facebook
         # NOTICE: check meta before calling me
-        sender = self.facebook.identifier(msg.envelope.sender)
-        meta = Meta(msg.meta)
+        sender = facebook.identifier(msg.envelope.sender)
+        meta = msg.meta
         if meta is None:
-            meta = self.facebook.meta(identifier=sender)
+            meta = facebook.meta(identifier=sender)
             if meta is None:
-                # TODO: query meta for sender from DIM network
-                #       (do it by application)
-                raise LookupError('failed to get meta for sender: %s' % sender)
+                # NOTICE: the application will query meta automatically
+                # save this message in a queue waiting sender's meta response
+                self.suspend_message(msg=msg)
+                # raise LookupError('failed to get meta for sender: %s' % sender)
+                return None
         else:
             # [Meta Protocol]
             # save meta for sender
-            if not self.facebook.save_meta(meta=meta, identifier=sender):
+            meta = Meta(meta)
+            if not facebook.save_meta(meta=meta, identifier=sender):
                 raise ValueError('save meta error: %s, %s' % (sender, meta))
         return super().verify_message(msg=msg)
 
@@ -188,28 +177,17 @@ class Messenger(Transceiver, ConnectionDelegate):
         return s_msg
 
     def decrypt_message(self, msg: SecureMessage) -> Optional[InstantMessage]:
-        # NOTICE: trim for group message before calling me
-        #         if there are more than 1 local user, check which is the group member
-        i_msg = super().decrypt_message(msg=msg)
-        # check: top-secret message
-        content = i_msg.content
-        if isinstance(content, ForwardContent):
-            # [Forward Protocol]
-            # do it again to drop the wrapper,
-            # the secret inside the content is the real message
-            r_msg = content.forward
-            s_msg = self.verify_message(msg=r_msg)
-            if s_msg is not None:
-                # verify OK, try to decrypt
-                secret = self.decrypt_message(msg=s_msg)
-                if secret is not None:
-                    # decrypt success
-                    return secret
-                # NOTICE: decrypt failed, not for you?
-                #         check content type in subclass, if it's a 'forward' message,
-                #         it means you are asked to re-pack and forward this message
-        return i_msg
+        # trim message
+        s_msg = self.__trim(msg=msg)
+        if s_msg is None:
+            # not for you?
+            raise LookupError('receiver error: %s' % msg)
+        # decrypt message
+        return super().decrypt_message(msg=s_msg)
 
+    #
+    #   InstantMessageDelegate
+    #
     def encrypt_content(self, content: Content, key: dict, msg: InstantMessage) -> bytes:
         password = SymmetricKey(key=key)
         assert password == key, 'irregular symmetric key: %s' % key
@@ -223,6 +201,22 @@ class Messenger(Transceiver, ConnectionDelegate):
                 content.data = None
         return super().encrypt_content(content=content, key=password, msg=msg)
 
+    def encrypt_key(self, key: dict, receiver: str, msg: InstantMessage) -> Optional[bytes]:
+        facebook = self.facebook
+        to = facebook.identifier(receiver)
+        pk = facebook.public_key_for_encryption(identifier=to)
+        if pk is None:
+            meta = facebook.meta(identifier=to)
+            if meta is None:
+                # save this message in a queue waiting receiver's meta response
+                self.suspend_message(msg=msg)
+                # raise LookupError('failed to get encrypt key for receiver: %s' % receiver)
+                return None
+        return super().encrypt_key(key=key, receiver=receiver, msg=msg)
+
+    #
+    #   SecureMessageDelegate
+    #
     def decrypt_content(self, data: bytes, key: dict, msg: SecureMessage) -> Optional[Content]:
         password = SymmetricKey(key=key)
         content = super().decrypt_content(data=data, key=password, msg=msg)
@@ -256,7 +250,7 @@ class Messenger(Transceiver, ConnectionDelegate):
         :param split:    if it's a group message, split it before sending out
         :return: True on success
         """
-        user = self.current_user
+        user = self.facebook.current_user
         assert user is not None, 'failed to get current user'
         i_msg = InstantMessage.new(content=content, sender=user.identifier, receiver=receiver)
         return self.send_message(msg=i_msg, callback=callback, split=split)
@@ -270,14 +264,15 @@ class Messenger(Transceiver, ConnectionDelegate):
         :param split:    if it's a group message, split it before sending out
         :return:         False on data/delegate error
         """
+        facebook = self.facebook
         # Send message (secured + certified) to target station
         s_msg = self.encrypt_message(msg=msg)
         r_msg = self.sign_message(msg=s_msg)
-        receiver = self.facebook.identifier(msg.envelope.receiver)
+        receiver = facebook.identifier(msg.envelope.receiver)
         ok = True
         if split and receiver.type.is_group():
             # split for each members
-            members = self.facebook.members(identifier=receiver)
+            members = facebook.members(identifier=receiver)
             if members is None or len(members) == 0:
                 # FIXME: query group members from sender
                 messages = None
@@ -294,13 +289,43 @@ class Messenger(Transceiver, ConnectionDelegate):
         else:
             ok = self.__send_message(msg=r_msg, callback=callback)
         # TODO: if OK, set iMsg.state = sending; else set iMsg.state = waiting
+
+        if not self.save_message(msg=msg):
+            return False
         return ok
 
     def __send_message(self, msg: ReliableMessage, callback: Callback) -> bool:
-        data = self.serialize_message(msg=msg)
         handler = MessageCallback(msg=msg, cb=callback)
+        data = self.serialize_message(msg=msg)
         return self.delegate.send_package(data=data, handler=handler)
 
+    #
+    #   Saving Message
+    #
+
+    @abstractmethod
+    def save_message(self, msg: InstantMessage) -> bool:
+        """
+        Save instant message
+
+        :param msg: instant message
+        :return: True on success
+        """
+        raise NotImplemented
+
+    @abstractmethod
+    def suspend_message(self, msg: Union[ReliableMessage, InstantMessage]):
+        """
+        Suspend reliable message for the sender's meta
+        Suspend instant message for the receiver's meta or group info
+
+        :param msg: message received from network / instant message to be sent
+        """
+        # NOTICE: this function is for Client
+        #         if the client cannot get verify/encrypt message for contact,
+        #         it means you should suspend it and query meta from DIM station first
+        raise NotImplemented
+
     #
     #   ConnectionDelegate
     #
@@ -311,148 +336,57 @@ class Messenger(Transceiver, ConnectionDelegate):
         :param data: message data
         :return: response message data
         """
+        # 1. deserialize message
         r_msg = self.deserialize_message(data=data)
-        response = self.__process_message(msg=r_msg)
+        if r_msg is None:
+            # no message received
+            return None
+        # 2. process message
+        response = self.process_reliable(msg=r_msg)
         if response is None:
             # nothing to response
             return None
-        # response to the sender
-        user = self.current_user
-        assert user is not None, 'failed to get current user'
-        sender = self.facebook.identifier(r_msg.envelope.sender)
+        # 3. pack response
+        facebook = self.facebook
+        sender = facebook.identifier(r_msg.envelope.sender)
+        receiver = facebook.identifier(r_msg.envelope.receiver)
+        user = self.__select(receiver=receiver)
+        if user is None:
+            # not for you?
+            # delivering message to other receiver?
+            user = facebook.current_user
         i_msg = InstantMessage.new(content=response, sender=user.identifier, receiver=sender)
         s_msg = self.encrypt_message(msg=i_msg)
-        msg_r = self.sign_message(msg=s_msg)
-        assert msg_r is not None, 'failed to response: %s' % i_msg
-        return self.serialize_message(msg=msg_r)
+        r_msg = self.sign_message(msg=s_msg)
+        assert r_msg is not None, 'failed to response: %s' % i_msg
+        # serialize message
+        return self.serialize_message(msg=r_msg)
 
-    def __is_empty(self, group: ID) -> bool:
-        """
-        Check whether group info empty (lost)
-
-        :param group: group ID
-        :return: True on members, owner not found
-        """
-        members = self.facebook.members(identifier=group)
-        if members is None or len(members) == 0:
-            return True
-        owner = self.facebook.owner(identifier=group)
-        if owner is None:
-            return True
-
-    def __check_group(self, content: Content, sender: ID) -> bool:
-        """
-        Check if it is a group message, and whether the group members info needs update
-
-        :param content: message content
-        :param sender:  message sender
-        :return: True on updating
-        """
-        group = self.facebook.identifier(content.group)
-        if group is None or group.is_broadcast:
-            # 1. personal message
-            # 2. broadcast message
-            return False
-        # check meta for new group ID
-        meta = self.facebook.meta(identifier=group)
-        if meta is None:
-            # NOTICE: if meta for group not found,
-            #         facebook should query it from DIM network automatically
-            # TODO: insert the message to a temporary queue to wait meta
-            raise LookupError('group meta not found: %s' % group)
-        # NOTICE: if the group info not found, and this is not an 'invite' command
-        #         query group info from the sender
-        needs_update = self.__is_empty(group=group)
-        if isinstance(content, InviteCommand):
-            # FIXME: can we trust this stranger?
-            #        may be we should keep this members list temporary,
-            #        and send 'query' to the owner immediately.
-            # TODO: check whether the members list is a full list,
-            #       it should contain the group owner(owner)
-            needs_update = False
-        if needs_update:
-            query = GroupCommand.query(group=group)
-            return self.send_content(content=query, receiver=sender)
-
-    def __process_message(self, msg: ReliableMessage) -> Optional[Content]:
-        # verify
+    def process_reliable(self, msg: ReliableMessage) -> Optional[Content]:
         s_msg = self.verify_message(msg=msg)
         if s_msg is None:
-            raise ValueError('failed to verify message: %s' % msg)
-        receiver = self.facebook.identifier(msg.envelope.receiver)
-        #
-        #  1. check broadcast
-        #
-        if receiver.type.is_group() and receiver.is_broadcast:
-            # if it's a grouped broadcast id, then split and deliver to everyone
-            return self.broadcast_message(msg=msg)
-        #
-        #  2. try to decrypt
-        #
-        i_msg = self.decrypt_message(msg=s_msg)
-        if i_msg is None:
-            # cannot decrypt this message, not for you?
-            # deliver to the receiver
-            return self.deliver_message(msg=msg)
-        #
-        #  3. check top-secret message
-        #
-        content = i_msg.content
-        if isinstance(content, ForwardContent):
-            # it's asking you to forward it
-            return self.forward_message(msg=content.forward)
-        #
-        #  4. check group
-        #
-        sender = self.facebook.identifier(msg.envelope.sender)
-        if self.__check_group(content=content, sender=sender):
-            pass
-        #
-        #  5. process
-        #
-        response = self.cpu().process(content=content, sender=sender, msg=i_msg)
-        if self.save_message(msg=i_msg):
-            return response
+            # waiting for sender's meta if not exists
+            return None
+        # TODO: override to check broadcast message before calling it
+        # TODO: override to deliver to the receiver when catch exception "receiver error ..."
+        return self.process_secure(msg=s_msg)
 
-    @abstractmethod
-    def save_message(self, msg: InstantMessage) -> bool:
-        """
-        Save instant message
+    def process_secure(self, msg: SecureMessage) -> Optional[Content]:
+        # try to decrypt
+        i_msg = self.decrypt_message(msg=msg)
+        assert i_msg is not None, 'failed to decrypt message: %s' % msg
+        # process it
+        return self.process_instant(msg=i_msg)
 
-        :param msg: instant message
-        :return: True on success
-        """
-        pass
-
-    @abstractmethod
-    def broadcast_message(self, msg: ReliableMessage) -> Optional[Content]:
-        """
-        Deliver message to everyone@everywhere, including all neighbours
-
-        :param msg: broadcast message
-        :return: receipt on success
-        """
-        pass
-
-    @abstractmethod
-    def deliver_message(self, msg: ReliableMessage) -> Optional[Content]:
-        """
-        Deliver message to the receiver, or broadcast to neighbours
-
-        :param msg: reliable message
-        :return: receipt on success
-        """
-        pass
-
-    @abstractmethod
-    def forward_message(self, msg: ReliableMessage) -> Optional[Content]:
-        """
-        Re-pack and deliver (Top-Secret) message to the real receiver
-
-        :param msg: top-secret message
-        :return: receipt on success
-        """
-        pass
+    def process_instant(self, msg: InstantMessage) -> Optional[Content]:
+        # TODO: override to check group
+        sender = self.facebook.identifier(string=msg.envelope.sender)
+        res = self.__cpu.process(content=msg.content, sender=sender, msg=msg)
+        if not self.save_message(msg=msg):
+            # error
+            return None
+        # TODO: override to filter the response
+        return res
 
 
 class MessageCallback(CompletionHandler):
